// LRFFile.cs
//
// Copyright (C) 2004  Peter Knowles
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.

using System;
using System.Text;
using System.IO;
using ICSharpCode.SharpZipLib.Zip.Compression;

namespace OpenLibrie.LRFLib {

  /// <summary>
  /// Represents a LRF file
  /// </summary>
  class LRFFile {

    /// <summary>
    /// Size of a WORD
    /// </summary>
    public const int WORD_SIZE = 2;

    /// <summary>
    /// Size of a DWORD
    /// </summary>
    public const int DWORD_SIZE = 4;

    /// <summary>
    /// Offset of length of compressed info block in LRF file
    /// </summary>
    public const int LRF_OFFSET_CINFOBLOCK_LEN = 0x4C;

    /// <summary>
    /// Offset of length of uncompressed info block in LRF file
    /// </summary>
    public const int LRF_OFFSET_UINFOBLOCK_LEN = 0x54;

    /// <summary>
    /// Offset of compressed info block data in LRF file
    /// </summary>
    public const int LRF_OFFSET_CINFOBLOCK_DATA = LRF_OFFSET_UINFOBLOCK_LEN + DWORD_SIZE;

    /// <summary>
    /// Offset of length of GIF data in LRF file
    /// </summary>
    public const int LRF_OFFSET_GIF_LEN = 0x50;

    /// <summary>
    /// Path of LRF file
    /// </summary>
    private string location;

    /// <summary>
    /// The uncompressed UTF-16 info block
    /// </summary>
    private byte[] bytesXMLInfoBlock;

    /// <summary>
    /// The GIF file data for the thumnail image
    /// </summary>
    private byte[] bytesGIFData;

    /// <summary>
    /// The full path of the LRF file
    /// </summary>
    public string Location {
      get { return location; }
      set { location = value; }
    }

    /// <summary>
    /// The GIF data for the LRF thumbnail image
    /// </summary>
    public byte[] GIFData {
      get { return bytesGIFData; }
      set { bytesGIFData = value; }
    }

    /// <summary>
    /// The file size of the LRF file
    /// </summary>
    private long fileSize;

    /// <summary>
    /// The file size of the LRF file
    /// </summary>
    public long FileSize {
      get { return fileSize; }
    }

    /// <summary>
    /// The uncompressed UTF-16 info block
    /// </summary>
    public byte[] XMLInfoBlock {
      get { return bytesXMLInfoBlock; }
      set { bytesXMLInfoBlock = value; }
    }

    /// <summary>
    /// Initializes a new instance of the LRFFile class, using data from the
    /// specified LRF file.
    /// </summary>
    /// <param name="filename">
    /// The name of the LRF file to read from.
    /// </param>
    public LRFFile(string filename) {
      this.location = filename;
      
      this.fileSize = new FileInfo(filename).Length;

      /* Used to store the whole LRF file as bytes */
      byte[] lrfBytes = new Byte[this.fileSize];

      #region ReadEntireLRFFile
      /* Read the entire LRF file into lrfBytes */
      byte[] tempBuffer = new Byte[4096];
      FileStream fs = new FileStream(filename, FileMode.Open);
      int numBytesRead = 0;
      int curPosition = 0;
      while (0 != (numBytesRead = fs.Read(tempBuffer, 0, 4096))) {
	Buffer.BlockCopy(tempBuffer, 0, lrfBytes, curPosition, numBytesRead);
	curPosition += numBytesRead;
      }
      fs.Close ();
      tempBuffer = null;
      #endregion

      #region ReadCompressedInfoBlock
      /* Allocate the temporary memory required for the Compressed Info Block */
      int infoLength = BitConverter.ToInt16(lrfBytes, LRF_OFFSET_CINFOBLOCK_LEN);
      byte[] bytesInfoData = new Byte[infoLength - DWORD_SIZE];

      /* Copy the Info block into the temporary buffer */
      Buffer.BlockCopy(lrfBytes, LRF_OFFSET_CINFOBLOCK_DATA, bytesInfoData, 0, (infoLength - DWORD_SIZE));
      #endregion

      #region ReadGIFData
      /* Allocate the required memory for the GIF */      
      int gifLength = BitConverter.ToInt32(lrfBytes, LRF_OFFSET_GIF_LEN);
      bytesGIFData = new Byte[gifLength];

      /* Copy the GIF data into bytesGIFData */
      Buffer.BlockCopy(lrfBytes, (LRF_OFFSET_UINFOBLOCK_LEN + infoLength), bytesGIFData, 0, gifLength);
      #endregion

      #region UncompressInfoBlock
      /* Allocate the required memory for the Uncompressed Info Block */
      int infoUncompressedLength = BitConverter.ToInt32(lrfBytes, LRF_OFFSET_UINFOBLOCK_LEN);
      bytesXMLInfoBlock = new byte[infoUncompressedLength];

      /* Uncompress the info block */
      Inflater i = new Inflater();
      i.SetInput(bytesInfoData);
      i.Inflate(bytesXMLInfoBlock);
      #endregion
    }

  }
  
}
